<?php

namespace TVn7\ForumBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;
use TVn7\UserBundle\Entity\Membre;

/**
 * Forum
 *
 * @ORM\Table(name="tdb_forum_forum")
 * @ORM\Entity(repositoryClass="TVn7\ForumBundle\Repository\ForumRepository")
 * @Gedmo\Loggable(logEntryClass="TVn7\LogBundle\Entity\AppsLog")
 */
class Forum
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=255, unique=true)
     * @Gedmo\Versioned
     */
    private $name;

    /**
     * @var string
     * @Gedmo\Slug(fields={"name"}, updatable=false)
     * @ORM\Column(name="slug", type="string", length=255, unique=true)
     */
    private $slug;

    /**
     * @var string
     *
     * @Assert\NotBlank()
     * @ORM\Column(name="description", type="text")
     * @Gedmo\Versioned
     */
    private $description;

    /**
     * @var array
     *
     * @ORM\ManyToMany(targetEntity="TVn7\UserBundle\Entity\Membre", inversedBy="forumSubscribe")
     * @ORM\JoinTable(name="tdb_forum_subscribers")
     */
    private $subscribers;

    /**
     * @ORM\OneToMany(targetEntity="TVn7\ForumBundle\Entity\Topic", mappedBy="forum", cascade={"persist"})
     */
    private $topics;

    /**
     *
     * @var boolean
     *
     * @ORM\Column(name="deleted", type="boolean", options={"default": 0})
     * @Gedmo\Versioned
     */
    private $deleted = false;

    /**
     *
     * @var boolean
     *
     * @ORM\Column(name="allow_new_post", type="boolean", options={"default": 1})
     * @Gedmo\Versioned
     */
    private $allow_new_post = true;

    /**
     *
     * @var boolean
     *
     * @ORM\Column(name="allow_new_comment", type="boolean", options={"default": 1})
     * @Gedmo\Versioned
     */
    private $allowNewReply = true;

    /**
     *
     * @var boolean
     *
     * @ORM\Column(name="visible", type="boolean", options={"default": 1})
     * @Gedmo\Versioned
     */
    private $visible = true;

    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     *
     * @return Forum
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->subscribers = new \Doctrine\Common\Collections\ArrayCollection();
        $this->topics = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Set slug
     *
     * @param string $slug
     *
     * @return Forum
     */
    public function setSlug($slug)
    {
        $this->slug = $slug;

        return $this;
    }

    /**
     * Get slug
     *
     * @return string
     */
    public function getSlug()
    {
        return $this->slug;
    }

    /**
     * Set desc
     *
     * @param string $desc
     *
     * @return Forum
     */
    public function setDescription($desc)
    {
        $this->description = $desc;

        return $this;
    }

    /**
     * Get desc
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Add subscriber
     *
     * @param Membre $subscriber
     *
     * @return Forum
     */
    public function addSubscriber(Membre $subscriber)
    {
        if(!$this->subscribers->contains($subscriber))
            $this->subscribers[] = $subscriber;

        return $this;
    }

    /**
     * Remove subscriber
     *
     * @param Membre $subscriber
     */
    public function removeSubscriber(Membre $subscriber)
    {
        $this->subscribers->removeElement($subscriber);
    }

    /**
     * Get subscribers
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getSubscribers()
    {
        return $this->subscribers;
    }

    /**
     * Add topic
     *
     * @param Topic $topic *
     * @return Forum
     */
    public function addTopic(Topic $topic)
    {
        $this->topics[] = $topic;

        return $this;
    }

    /**
     * Remove topic
     *
     * @param Topic $topic
     */
    public function removeTopic(Topic $topic)
    {
        $this->topics->removeElement($topic);
    }

    /**
     * Get topics
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getTopics()
    {
        return $this->topics;
    }

    /**
     * Set deleted
     *
     * @param boolean $deleted
     *
     * @return Forum
     */
    public function setDeleted($deleted)
    {
        $this->deleted = $deleted;

        return $this;
    }

    /**
     * Get deleted
     *
     * @return boolean
     */
    public function isDeleted()
    {
        return $this->deleted;
    }

    /**
     * Set allowNewPost
     *
     * @param boolean $allowNewPost
     *
     * @return Forum
     */
    public function setAllowNewPost($allowNewPost)
    {
        $this->allow_new_post = $allowNewPost;

        return $this;
    }

    /**
     * Get allowNewPost
     *
     * @return boolean
     */
    public function isAllowNewPost()
    {
        return $this->allow_new_post;
    }

    /**
     * Set allowNewReply
     *
     * @param boolean $allowNewReply
     *
     * @return Forum
     */
    public function setAllowNewReply($allowNewReply)
    {
        $this->allowNewReply = $allowNewReply;

        return $this;
    }

    /**
     * Get allowNewComment
     *
     * @return boolean
     */
    public function isAllowNewReply()
    {
        return $this->allowNewReply;
    }

    /**
     * Set visible
     *
     * @param boolean $visible
     *
     * @return Forum
     */
    public function setVisible($visible)
    {
        $this->visible = $visible;

        return $this;
    }

    /**
     * Get visible
     *
     * @return boolean
     */
    public function isVisible()
    {
        return $this->visible;
    }

    /**
     * Get comments
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getUndeletedTopics()
    {
        return $this->getTopics()->filter(function ($topic) {
            /** @var Topic $topic */
            return !$topic->isDeleted();
        });
    }
    
    public function __toString() {
        return $this->getName();
    }

}
